'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /* eslint no-fallthrough: off */


var _dateArithmetic = require('date-arithmetic');

var _dateArithmetic2 = _interopRequireDefault(_dateArithmetic);

var _localizer = require('../localizer');

var _localizer2 = _interopRequireDefault(_localizer);

var _momentTimezone = require('moment-timezone');

var _momentTimezone2 = _interopRequireDefault(_momentTimezone);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var MILLI = {
  seconds: 1000,
  minutes: 1000 * 60,
  hours: 1000 * 60 * 60,
  day: 1000 * 60 * 60 * 24
};

var DATE_WITHOUT_TIMEZONE_FORMAT = 'YYYY-MM-DD HH:mm:ss';

var MONTHS = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11];

var dates = _extends({}, _dateArithmetic2.default, {

  SECONDS_IN_DAY: 24 * 60 * 60,

  MINUTES_IN_DAY: 1440,

  monthsInYear: function monthsInYear(year) {
    var date = new Date(year, 0, 1);

    return MONTHS.map(function (i) {
      return dates.month(date, i);
    });
  },
  firstVisibleDay: function firstVisibleDay(date, culture) {
    var firstOfMonth = dates.startOf(date, 'month');

    return dates.startOf(firstOfMonth, 'week', _localizer2.default.startOfWeek(culture));
  },
  lastVisibleDay: function lastVisibleDay(date, culture) {
    var endOfMonth = dates.endOf(date, 'month');

    return dates.endOf(endOfMonth, 'week', _localizer2.default.startOfWeek(culture));
  },
  visibleDays: function visibleDays(date, culture) {
    var current = dates.firstVisibleDay(date, culture),
        last = dates.lastVisibleDay(date, culture),
        days = [];

    while (dates.lte(current, last, 'day')) {
      days.push(current);
      current = dates.add(current, 1, 'day');
    }

    return days;
  },
  ceil: function ceil(date, unit) {
    var floor = dates.startOf(date, unit);

    return dates.eq(floor, date) ? floor : dates.add(floor, 1, unit);
  },
  range: function range(start, end) {
    var unit = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'day';

    var current = start,
        days = [];

    while (dates.lte(current, end, unit)) {
      days.push(current);
      current = dates.add(current, 1, unit);
    }

    return days;
  },
  merge: function merge(date, time) {
    if (time == null && date == null) return null;

    if (time == null) time = new Date();
    if (date == null) date = new Date();

    date = dates.startOf(date, 'day');
    date = dates.hours(date, dates.hours(time));
    date = dates.minutes(date, dates.minutes(time));
    date = dates.seconds(date, dates.seconds(time));
    return dates.milliseconds(date, dates.milliseconds(time));
  },
  sameDate: function sameDate(date, date2) {
    return (0, _momentTimezone2.default)(date).isSame(date2, 'date');
  },
  sameMonth: function sameMonth(dateA, dateB) {
    return dates.eq(dateA, dateB, 'month');
  },
  isToday: function isToday(date, timezone) {
    return dates.eq(date, dates.today(timezone), 'day');
  },
  eqTime: function eqTime(dateA, dateB) {
    return dates.hours(dateA) === dates.hours(dateB) && dates.minutes(dateA) === dates.minutes(dateB) && dates.seconds(dateA) === dates.seconds(dateB);
  },
  isJustDate: function isJustDate(date) {
    return dates.hours(date) === 0 && dates.minutes(date) === 0 && dates.seconds(date) === 0 && dates.milliseconds(date) === 0;
  },
  duration: function duration(start, end, unit, firstOfWeek) {
    if (unit === 'day') unit = 'date';
    return Math.abs(dates[unit](start, undefined, firstOfWeek) - dates[unit](end, undefined, firstOfWeek));
  },
  diff: function diff(dateA, dateB, unit) {
    if (!unit || unit === 'milliseconds') return Math.abs(+dateA - +dateB);

    // disabling DST for grid rendering --->>>
    var tz = _momentTimezone2.default.tz.guess();
    var tzOffsetDateA = _momentTimezone2.default.tz.zone(tz).offset(Date.parse(dateA));
    var tzOffsetDateB = _momentTimezone2.default.tz.zone(tz).offset(Date.parse(dateB));
    var dstOffset = tzOffsetDateA - tzOffsetDateB;

    if (dstOffset < 0) {
      var dateANorm = (0, _momentTimezone2.default)(dateA).add(-dstOffset, 'minutes').toDate();
      return Math.round(Math.abs(+dates.startOf(dateANorm, unit) / MILLI[unit] - +dates.startOf(dateB, unit) / MILLI[unit]));
    } else {
      var dateBNorm = (0, _momentTimezone2.default)(dateB).add(dstOffset, 'minutes').toDate();
      return Math.round(Math.abs(+dates.startOf(dateA, unit) / MILLI[unit] - +dates.startOf(dateBNorm, unit) / MILLI[unit]));
    }
    // disabling DST for grid rendering <<<---

    // the .round() handles an edge case
    // with DST where the total won't be exact
    // since one day in the range may be shorter/longer by an hour
    return Math.round(Math.abs(+dates.startOf(dateA, unit) / MILLI[unit] - +dates.startOf(dateB, unit) / MILLI[unit]));
  },
  getTimezoneOffset: function getTimezoneOffset(date, timezone) {
    return _momentTimezone2.default.tz.zone(timezone).offset(Date.parse(date));
  },
  getLocalTimezone: function getLocalTimezone() {
    return _momentTimezone2.default.tz.guess();
  },
  getTZNormalizedEndDate: function getTZNormalizedEndDate(end, offset) {
    var endDate = end;
    if ((0, _momentTimezone2.default)(end).seconds() !== 0) {
      endDate = (0, _momentTimezone2.default)(endDate).seconds(0);
    }
    return (0, _momentTimezone2.default)(endDate).add(offset, 'minutes').toDate();
  },
  total: function total(date, unit) {
    var ms = date.getTime(),
        div = 1;

    switch (unit) {
      case 'week':
        div *= 7;
      case 'day':
        div *= 24;
      case 'hours':
        div *= 60;
      case 'minutes':
        div *= 60;
      case 'seconds':
        div *= 1000;
    }

    return ms / div;
  },
  week: function week(date) {
    var d = new Date(date);
    d.setHours(0, 0, 0);
    d.setDate(d.getDate() + 4 - (d.getDay() || 7));
    return Math.ceil(((d - new Date(d.getFullYear(), 0, 1)) / 8.64e7 + 1) / 7);
  },
  today: function today(timezone) {
    return timezone ? (0, _momentTimezone2.default)((0, _momentTimezone2.default)().tz(timezone).startOf('day').format(DATE_WITHOUT_TIMEZONE_FORMAT)).toDate() : dates.startOf(new Date(), 'day');
  },
  now: function now(timezone) {
    return timezone ? (0, _momentTimezone2.default)((0, _momentTimezone2.default)().tz(timezone).format(DATE_WITHOUT_TIMEZONE_FORMAT)).toDate() : new Date();
  },
  yesterday: function yesterday() {
    return dates.add(dates.startOf(new Date(), 'day'), -1, 'day');
  },
  tomorrow: function tomorrow() {
    return dates.add(dates.startOf(new Date(), 'day'), 1, 'day');
  }
});

exports.default = dates;
module.exports = exports['default'];